package com.google.inject;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import static java.lang.annotation.ElementType.ANNOTATION_TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 * Annotates annotations which are used for scoping. Only one such annotation
 * may apply to a single implementation class. You must also annotate scope
 * annotations with {@code @Retention(RUNTIME)}. For example:
 *
 * <pre>
 *   {@code @}Retention(RUNTIME)
 *   {@code @}Target(TYPE, METHOD)
 *   {@code @}ScopeAnnotation
 *   public {@code @}interface SessionScoped {}
 * </pre>
 */
@Target(ANNOTATION_TYPE)
@Retention(RUNTIME)
public @interface ScopeAnnotation {
}
