package com.google.inject.binder;

import com.google.inject.Key;
import com.google.inject.Provider;
import com.google.inject.TypeLiteral;

import java.lang.reflect.Constructor;

/**
 * See the EDSL examples at {@link com.google.inject.Binder}.
 *
 */
public interface LinkedBindingBuilder<T> extends ScopedBindingBuilder {

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    ScopedBindingBuilder to(Class<? extends T> implementation);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    ScopedBindingBuilder to(TypeLiteral<? extends T> implementation);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    ScopedBindingBuilder to(Key<? extends T> targetKey);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     *
     * @see com.google.inject.Injector#injectMembers
     */
    void toInstance(T instance);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     *
     * @see com.google.inject.Injector#injectMembers
     */
    ScopedBindingBuilder toProvider(Provider<? extends T> provider);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     *
     * @see com.google.inject.Injector#injectMembers
     */
    ScopedBindingBuilder toProvider(javax.inject.Provider<? extends T> provider);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    ScopedBindingBuilder toProvider(Class<? extends javax.inject.Provider<? extends T>> providerType);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    ScopedBindingBuilder toProvider(TypeLiteral<? extends javax.inject.Provider<? extends T>> providerType);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    ScopedBindingBuilder toProvider(Key<? extends javax.inject.Provider<? extends T>> providerKey);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    <S extends T> ScopedBindingBuilder toConstructor(Constructor<S> constructor);

    /**
     * See the EDSL examples at {@link com.google.inject.Binder}.
     */
    <S extends T> ScopedBindingBuilder toConstructor(Constructor<S> constructor, TypeLiteral<? extends S> type);
}
