package com.google.inject.internal.util;

import static java.util.concurrent.TimeUnit.MILLISECONDS;

import com.google.common.base.Stopwatch;
import java.util.logging.Logger;

/**
 * A continuously timing stopwatch that is used for simple performance monitoring.
 */
public final class ContinuousStopwatch {
    private final Logger logger = Logger.getLogger(ContinuousStopwatch.class.getName());
    private final Stopwatch stopwatch;

    /**
     * Constructs a ContinuousStopwatch, which will start timing immediately after construction.
     *
     * @param stopwatch the internal stopwatch used by ContinuousStopwatch
     */
    public ContinuousStopwatch(Stopwatch stopwatch) {
        this.stopwatch = stopwatch;
        reset();
    }

    /** Resets and returns elapsed time in milliseconds. */
    public long reset() {
        long elapsedTimeMs = stopwatch.elapsed(MILLISECONDS);
        stopwatch.reset();
        stopwatch.start();
        return elapsedTimeMs;
    }

    /** Resets and logs elapsed time in milliseconds. */
    public void resetAndLog(String label) {
        logger.fine(label + ": " + reset() + "ms");
    }
}
