package com.google.inject.spi;

import com.google.inject.Binding;

/**
 * No-op visitor for subclassing. All interface methods simply delegate to
 * {@link #visitOther(Element)}, returning its result.
 *
 * @param <V> any type to be returned by the visit method. Use {@link Void} with
 *            {@code return null} if no return type is needed.
 */
public abstract class DefaultElementVisitor<V> implements ElementVisitor<V> {

    /**
     * Default visit implementation. Returns {@code null}.
     */
    protected V visitOther(Element element) {
        return null;
    }

    @Override
    public V visit(Message message) {
        return visitOther(message);
    }

    @Override
    public <T> V visit(Binding<T> binding) {
        return visitOther(binding);
    }

    @Override
    public V visit(ScopeBinding scopeBinding) {
        return visitOther(scopeBinding);
    }

    @Override
    public V visit(TypeConverterBinding typeConverterBinding) {
        return visitOther(typeConverterBinding);
    }

    @Override
    public <T> V visit(ProviderLookup<T> providerLookup) {
        return visitOther(providerLookup);
    }

    @Override
    public V visit(InjectionRequest<?> injectionRequest) {
        return visitOther(injectionRequest);
    }

    @Override
    public V visit(StaticInjectionRequest staticInjectionRequest) {
        return visitOther(staticInjectionRequest);
    }

    @Override
    public V visit(PrivateElements privateElements) {
        return visitOther(privateElements);
    }

    @Override
    public <T> V visit(MembersInjectorLookup<T> lookup) {
        return visitOther(lookup);
    }

    @Override
    public V visit(TypeListenerBinding binding) {
        return visitOther(binding);
    }

    @Override
    public V visit(ProvisionListenerBinding binding) {
        return visitOther(binding);
    }

    @Override
    public V visit(DisableCircularProxiesOption option) {
        return visitOther(option);
    }

    @Override
    public V visit(RequireExplicitBindingsOption option) {
        return visitOther(option);
    }

    @Override
    public V visit(RequireAtInjectOnConstructorsOption option) {
        return visitOther(option);
    }

    @Override
    public V visit(RequireExactBindingAnnotationsOption option) {
        return visitOther(option);
    }

    @Override
    public V visit(ModuleAnnotatedMethodScannerBinding binding) {
        return visitOther(binding);
    }
}
