package com.google.inject.spi;

import static com.google.common.base.Preconditions.checkNotNull;

import com.google.inject.Binder;
import com.google.inject.internal.Errors;

/**
 * Represents a call to {@link Binder#scanModulesForAnnotatedMethods} in a module.
 */
public final class ModuleAnnotatedMethodScannerBinding implements Element {
    private final Object source;
    private final ModuleAnnotatedMethodScanner scanner;

    public ModuleAnnotatedMethodScannerBinding(Object source, ModuleAnnotatedMethodScanner scanner) {
        this.source = checkNotNull(source, "source");
        this.scanner = checkNotNull(scanner, "scanner");
    }

    @Override
    public Object getSource() {
        return source;
    }

    public ModuleAnnotatedMethodScanner getScanner() {
        return scanner;
    }

    @Override
    public <T> T acceptVisitor(ElementVisitor<T> visitor) {
        return visitor.visit(this);
    }

    @Override
    public void applyTo(Binder binder) {
        binder.withSource(getSource()).scanModulesForAnnotatedMethods(scanner);
    }

    @Override
    public String toString() {
        return scanner
                + " which scans for "
                + scanner.annotationClasses()
                + " (bound at "
                + Errors.convert(source)
                + ")";
    }
}
