package com.google.inject.spi;

import com.google.inject.Binding;
import com.google.inject.Key;

/**
 * A binding to a provider key. To resolve injections, the provider key is first resolved, then that
 * provider's {@code get} method is invoked.
 */
public interface ProviderKeyBinding<T> extends Binding<T> {

    /**
     * Returns the key used to resolve the provider's binding. That binding can be retrieved from an
     * injector using {@link com.google.inject.Injector#getBinding(Key)
     * Injector.getBinding(providerKey)}
     */
    Key<? extends javax.inject.Provider<? extends T>> getProviderKey();

}