package com.google.inject.example;

import static org.junit.Assert.assertTrue;

public class ClientServiceWithFactories {

// 58 lines

    public static void main(String[] args) {
        new ClientServiceWithFactories().testClient();
    }

    public void testClient() {
        Service previous = ServiceFactory.getInstance();
        try {
            final MockService mock = new MockService();
            ServiceFactory.setInstance(mock);
            Client client = new Client();
            client.go();
            assertTrue(mock.isGone());
        } finally {
            ServiceFactory.setInstance(previous);
        }
    }

    public interface Service {
        void go();
    }

    public static class ServiceImpl implements Service {
        public void go() {
            // ...
        }
    }

    public static class ServiceFactory {

        private static Service instance = new ServiceImpl();

        private ServiceFactory() {
        }

        public static Service getInstance() {
            return instance;
        }

        public static void setInstance(Service service) {
            instance = service;
        }
    }

    public static class Client {

        public void go() {
            Service service = ServiceFactory.getInstance();
            service.go();
        }
    }

    public static class MockService implements Service {

        private boolean gone = false;

        public void go() {
            gone = true;
        }

        public boolean isGone() {
            return gone;
        }
    }
}
