package com.google.inject.example;

import static org.junit.Assert.assertTrue;

import com.google.inject.AbstractModule;
import com.google.inject.CreationException;
import com.google.inject.Guice;
import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Scopes;

public class ClientServiceWithGuice {

// 48 lines

    public static void main(String[] args) throws CreationException {
        new ClientServiceWithGuice().testClient();
        Injector injector = Guice.createInjector(new MyModule());
        Client client = injector.getInstance(Client.class);
    }

    public void testClient() {
        MockService mock = new MockService();
        Client client = new Client(mock);
        client.go();
        assertTrue(mock.isGone());
    }

    public interface Service {
        void go();
    }

    public static class ServiceImpl implements Service {
        public void go() {
            // ...
        }
    }

    public static class MyModule extends AbstractModule {
        protected void configure() {
            bind(Service.class).to(ServiceImpl.class).in(Scopes.SINGLETON);
        }
    }

    public static class Client {

        private final Service service;

        @Inject
        public Client(Service service) {
            this.service = service;
        }

        public void go() {
            service.go();
        }
    }

    public static class MockService implements Service {

        private boolean gone = false;

        public void go() {
            gone = true;
        }

        public boolean isGone() {
            return gone;
        }
    }
}
