package com.google.inject.internal;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import com.google.inject.TypeLiteral;

import org.junit.Test;
import java.lang.reflect.Type;
import java.util.Map;
import java.util.Set;

public class MoreTypesTest {

    @Test
    public void testParameterizedTypeToString() {
        TypeLiteral<Inner<String>> innerString = new TypeLiteral<>() {
        };
        assertEquals("com.google.inject.internal.MoreTypesTest$Inner<java.lang.String>",
                MoreTypes.typeToString(innerString.getType()));

        TypeLiteral<Set<Inner<Integer>>> mapInnerInteger = new TypeLiteral<>() {
        };
        assertEquals("java.util.Set<com.google.inject.internal.MoreTypesTest$Inner<java.lang.Integer>>",
                MoreTypes.typeToString(mapInnerInteger.getType()));

        TypeLiteral<Map<Inner<Long>, Set<Inner<Long>>>> mapInnerLongToSetInnerLong =
                new TypeLiteral<>() {
                };
        assertEquals("java.util.Map<com.google.inject.internal.MoreTypesTest$Inner<java.lang.Long>, "
                        + "java.util.Set<com.google.inject.internal.MoreTypesTest$Inner<java.lang.Long>>>",
                MoreTypes.typeToString(mapInnerLongToSetInnerLong.getType()));
    }

    @Test
    public <T> void testEquals_typeVariable() throws Exception {
        Type type = getClass().getMethod("testEquals_typeVariable").getTypeParameters()[0];
        assertTrue(MoreTypes.equals(new TypeLiteral<T>() {
        }.getType(), type));
    }

    public static class Inner<T> {
    }
}
