package com.google.inject.util;

import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertSame;
import com.google.common.base.Objects;
import com.google.common.testing.EqualsTester;
import com.google.inject.Provider;

import org.junit.Test;
import javax.inject.Inject;

/**
 * Unit tests for {@link Providers}.
 */
public class ProvidersTest {

    @Test
    public void testOfInstance() {
        String foo = "foo";
        Provider<String> p = Providers.of(foo);
        assertSame(foo, p.get());
        assertSame(foo, p.get());
    }

    @Test
    public void testOfNull() {
        Provider<String> p = Providers.of(null);
        assertNull(p.get());
    }

    @Test
    public void testOfEquality() {
        new EqualsTester()
                .addEqualityGroup(
                        Providers.of(null),
                        Providers.of(null))
                .addEqualityGroup(
                        Providers.of("Hello"),
                        Providers.of("Hello"))
                .testEquals();
    }

    private static class JavaxProvider implements javax.inject.Provider<Integer> {
        private final int value;

        public JavaxProvider(int value) {
            this.value = value;
        }

        public Integer get() {
            return value;
        }

        @Override
        public int hashCode() {
            return Objects.hashCode(value);
        }

        @Override
        public boolean equals(Object obj) {
            return (obj instanceof JavaxProvider) && (value == ((JavaxProvider) obj).value);
        }
    }

    private static class JavaxProviderWithDependencies implements javax.inject.Provider<Integer> {
        private int value;

        @Inject
        void setValue(int value) {
            this.value = value;
        }

        public Integer get() {
            return value;
        }

        @Override
        public int hashCode() {
            return 42;
        }

        @Override
        public boolean equals(Object obj) {
            return (obj instanceof JavaxProviderWithDependencies);
        }
    }
}
